/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements. See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership. The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the  "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/*
 * $Id:$
 */
package org.apache.xml.utils;

import javax.xml.transform.ErrorListener;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;

import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * Makes a SAX <code>ErrorHandler</code> that acts as a proxy for a JAXP
 * <code>ErrorHandler</code>.
 */
public class ErrorHandlerProxyForErrorListener implements ErrorHandler {
    /**
     * The contained JAXP <code>ErrorListener</code>
     */
    private ErrorListener m_listener;

    /**
     * Specifies whether an exception should be reported to the contained
     * <code>ErrorListener</code> as a <code>TransformerConfigurationException</code>
     * or a <code>TransformerException</code>.
     */
    private boolean m_reportAsConfigurationException;

    /**
     * Construct a SAX <code>ErrorHandler</code> that will wrap any reported
     * <code>SAXParseException</code> in a JAXP
     * <code>TransformerException</code>, and pass it to the specified JAXP
     * <code>ErrorListener</code> for handling.
     *
     * @param listener The contained JAXP <code>ErrorListener</code>
     * @param reportAsConfigurationException Specifies whether the reported
     *            exception should be wrapped in a
     *            <code>TransformerException</code> or a
     *            <code>TransformerConfigurationException</code>
     */
    public ErrorHandlerProxyForErrorListener(ErrorListener listener,
                   boolean resportAsConfigurationException) {
        m_listener = listener;
        m_reportAsConfigurationException = resportAsConfigurationException;
    }
    
    /**
     * Passes the reported warning to the enclosed <code>ErrorListener</code>.
     * <p>This method will throw a <code>SAXException</code> if and only if the
     * enclosed <code>ErrorListener</code> throws a
     * <code>TransformerException</code>.
     *
     * @param se An exception representing the warning
     * @throws SAXException
     */
    public void warning(SAXParseException se) throws SAXException {
        try {
           m_listener.warning(createWrapperException(se));
        } catch (TransformerException te) {
           throw new ReportedSAXException(te);
        }
    }

    /**
     * Passes the reported error to the enclosed <code>ErrorListener</code>
     * <p>This method will throw a <code>SAXException</code> if and only if the
     * enclosed <code>ErrorListener</code> throws a
     * <code>TransformerException</code>.
     *
     * @param se An exception representing the error
     * @throws SAXException
     */
    public void error(SAXParseException se) throws SAXException {
        try {
           m_listener.error(createWrapperException(se));
        } catch (TransformerException te) {
           throw new ReportedSAXException(te);
        }
    }

    /**
     * Passes the reported fatal error to the enclosed
     * <code>ErrorListener</code>.
     * <p>A fatal error is regarded as fatal to the XSLT processor, regardless
     * of whether the enclosed <code>ErrorListener</code> actually throws an
     * an exception in response to the report of the fatal error.
     *
     * @param se An exception representing the fatal error
     * @throws SAXException
     */
    public void fatalError(SAXParseException se) throws SAXException {
        TransformerException wrappedException = createWrapperException(se);
        try {
           m_listener.fatalError(wrappedException);
        } catch (TransformerException te) {
           throw new ReportedSAXException(te);
        }
        throw new ReportedSAXException(wrappedException);
    }

    /**
     * Wraps the specified <code>SAXParseException</code> in a
     * <code>TransformerException</code> or a
     * <code>TransformerConfigurationException</code>, as specified by the
     * constructor, {@link #ErrorHandlerProxyForErrorListener(javax.xml.transform.ErrorListener,boolean)}
     *
     * @param se The reported exception
     * @return The wrapping exception
     */
    private TransformerException createWrapperException(SAXParseException se) {
        return m_reportAsConfigurationException
                   ? new TransformerConfigurationException(se.getMessage(), se)
                   : new TransformerException(se.getMessage(), se);
    }
}
